@@ -39,22 +39,37 @@ extern "C"
     typedef unsigned __int32 uint32_t;
     typedef unsigned __int64 uint64_t;
 
-#ifdef plist_EXPORTS
-#define PLIST_API  __declspec( dllexport )
-#else
-#define PLIST_API  __declspec( dllimport )
-#endif
-#define DEPRECATED(x) __declspec(deprecated(x))
 #else
 #include <stdint.h>
-#define PLIST_API
-#ifdef __GNUC__
-#define DEPRECATED(x) __attribute__((deprecated(x)))
-#elif defined(_MSC_VER)
-#else
-#define DEPRECATED(x)
-#pragma message("WARNING: You need to implement DEPRECATED for this compiler")
 #endif
+
+#ifdef __llvm__
+  #if defined(__has_extension)
+    #if (__has_extension(attribute_deprecated_with_message))
+      #ifndef PLIST_WARN_DEPRECATED
+        #define PLIST_WARN_DEPRECATED(x) __attribute__((deprecated(x)))
+      #endif
+    #else
+      #ifndef PLIST_WARN_DEPRECATED
+        #define PLIST_WARN_DEPRECATED(x) __attribute__((deprecated))
+      #endif
+    #endif
+  #else
+    #ifndef PLIST_WARN_DEPRECATED
+      #define PLIST_WARN_DEPRECATED(x) __attribute__((deprecated))
+    #endif
+  #endif
+#elif (__GNUC__ > 4 || (__GNUC__ == 4 && (__GNUC_MINOR__ >= 5)))
+  #ifndef PLIST_WARN_DEPRECATED
+    #define PLIST_WARN_DEPRECATED(x) __attribute__((deprecated(x)))
+  #endif
+#elif defined(_MSC_VER)
+  #ifndef PLIST_WARN_DEPRECATED
+    #define PLIST_WARN_DEPRECATED(x) __declspec(deprecated(x))
+  #endif
+#else
+  #define PLIST_WARN_DEPRECATED(x)
+  #pragma message("WARNING: You need to implement DEPRECATED for this compiler")
 #endif
 
 #include <sys/types.h>
@@ -108,7 +123,7 @@ extern "C"
      * @return the created plist
      * @sa #plist_type
      */
-    PLIST_API plist_t plist_new_dict(void);
+    plist_t plist_new_dict(void);
 
     /**
      * Create a new root plist_t type #PLIST_ARRAY
@@ -116,7 +131,7 @@ extern "C"
      * @return the created plist
      * @sa #plist_type
      */
-    PLIST_API plist_t plist_new_array(void);
+    plist_t plist_new_array(void);
 
     /**
      * Create a new plist_t type #PLIST_STRING
@@ -125,7 +140,7 @@ extern "C"
      * @return the created item
      * @sa #plist_type
      */
-    PLIST_API plist_t plist_new_string(const char *val);
+    plist_t plist_new_string(const char *val);
 
     /**
      * Create a new plist_t type #PLIST_BOOLEAN
@@ -134,7 +149,7 @@ extern "C"
      * @return the created item
      * @sa #plist_type
      */
-    PLIST_API plist_t plist_new_bool(uint8_t val);
+    plist_t plist_new_bool(uint8_t val);
 
     /**
      * Create a new plist_t type #PLIST_UINT
@@ -143,7 +158,7 @@ extern "C"
      * @return the created item
      * @sa #plist_type
      */
-    PLIST_API plist_t plist_new_uint(uint64_t val);
+    plist_t plist_new_uint(uint64_t val);
 
     /**
      * Create a new plist_t type #PLIST_REAL
@@ -152,7 +167,7 @@ extern "C"
      * @return the created item
      * @sa #plist_type
      */
-    PLIST_API plist_t plist_new_real(double val);
+    plist_t plist_new_real(double val);
 
     /**
      * Create a new plist_t type #PLIST_DATA
@@ -162,7 +177,7 @@ extern "C"
      * @return the created item
      * @sa #plist_type
      */
-    PLIST_API plist_t plist_new_data(const char *val, uint64_t length);
+    plist_t plist_new_data(const char *val, uint64_t length);
 
     /**
      * Create a new plist_t type #PLIST_DATE
@@ -172,7 +187,7 @@ extern "C"
      * @return the created item
      * @sa #plist_type
      */
-    PLIST_API plist_t plist_new_date(int32_t sec, int32_t usec);
+    plist_t plist_new_date(int32_t sec, int32_t usec);
 
     /**
      * Create a new plist_t type #PLIST_UID
@@ -181,14 +196,14 @@ extern "C"
      * @return the created item
      * @sa #plist_type
      */
-    PLIST_API plist_t plist_new_uid(uint64_t val);
+    plist_t plist_new_uid(uint64_t val);
 
     /**
      * Destruct a plist_t node and all its children recursively
      *
      * @param plist the plist to free
      */
-    PLIST_API void plist_free(plist_t plist);
+    void plist_free(plist_t plist);
 
     /**
      * Return a copy of passed node and it's children
@@ -196,7 +211,7 @@ extern "C"
      * @param node the plist to copy
      * @return copied plist
      */
-    PLIST_API plist_t plist_copy(plist_t node);
+    plist_t plist_copy(plist_t node);
 
 
     /********************************************
@@ -211,7 +226,7 @@ extern "C"
      * @param node the node of type #PLIST_ARRAY
      * @return size of the #PLIST_ARRAY node
      */
-    PLIST_API uint32_t plist_array_get_size(plist_t node);
+    uint32_t plist_array_get_size(plist_t node);
 
     /**
      * Get the nth item in a #PLIST_ARRAY node.
@@ -220,7 +235,7 @@ extern "C"
      * @param n the index of the item to get. Range is [0, array_size[
      * @return the nth item or NULL if node is not of type #PLIST_ARRAY
      */
-    PLIST_API plist_t plist_array_get_item(plist_t node, uint32_t n);
+    plist_t plist_array_get_item(plist_t node, uint32_t n);
 
     /**
      * Get the index of an item. item must be a member of a #PLIST_ARRAY node.
@@ -228,43 +243,43 @@ extern "C"
      * @param node the node
      * @return the node index
      */
-    PLIST_API uint32_t plist_array_get_item_index(plist_t node);
+    uint32_t plist_array_get_item_index(plist_t node);
 
     /**
      * Set the nth item in a #PLIST_ARRAY node.
      * The previous item at index n will be freed using #plist_free
      *
      * @param node the node of type #PLIST_ARRAY
-     * @param item the new item at index n
+     * @param item the new item at index n. The array is responsible for freeing item when it is no longer needed.
      * @param n the index of the item to get. Range is [0, array_size[. Assert if n is not in range.
      */
-    PLIST_API void plist_array_set_item(plist_t node, plist_t item, uint32_t n);
+    void plist_array_set_item(plist_t node, plist_t item, uint32_t n);
 
     /**
      * Append a new item at the end of a #PLIST_ARRAY node.
      *
      * @param node the node of type #PLIST_ARRAY
-     * @param item the new item
+     * @param item the new item. The array is responsible for freeing item when it is no longer needed.
      */
-    PLIST_API void plist_array_append_item(plist_t node, plist_t item);
+    void plist_array_append_item(plist_t node, plist_t item);
 
     /**
      * Insert a new item at position n in a #PLIST_ARRAY node.
      *
      * @param node the node of type #PLIST_ARRAY
-     * @param item the new item to insert
+     * @param item the new item to insert. The array is responsible for freeing item when it is no longer needed.
      * @param n The position at which the node will be stored. Range is [0, array_size[. Assert if n is not in range.
      */
-    PLIST_API void plist_array_insert_item(plist_t node, plist_t item, uint32_t n);
+    void plist_array_insert_item(plist_t node, plist_t item, uint32_t n);
 
     /**
      * Remove an existing position in a #PLIST_ARRAY node.
-     * Removed position will be freed using #plist_free
+     * Removed position will be freed using #plist_free.
      *
      * @param node the node of type #PLIST_ARRAY
      * @param n The position to remove. Range is [0, array_size[. Assert if n is not in range.
      */
-    PLIST_API void plist_array_remove_item(plist_t node, uint32_t n);
+    void plist_array_remove_item(plist_t node, uint32_t n);
 
     /********************************************
      *                                          *
@@ -278,43 +293,46 @@ extern "C"
      * @param node the node of type #PLIST_DICT
      * @return size of the #PLIST_DICT node
      */
-    PLIST_API uint32_t plist_dict_get_size(plist_t node);
+    uint32_t plist_dict_get_size(plist_t node);
 
     /**
-     * Create iterator of a #PLIST_DICT node.
-     * The allocated iterator shoult be freed with tandard free function
+     * Create an iterator of a #PLIST_DICT node.
+     * The allocated iterator should be freed with the standard free function.
      *
      * @param node the node of type #PLIST_DICT
      * @param iter iterator of the #PLIST_DICT node
      */
-    PLIST_API void plist_dict_new_iter(plist_t node, plist_dict_iter *iter);
+    void plist_dict_new_iter(plist_t node, plist_dict_iter *iter);
 
     /**
      * Increment iterator of a #PLIST_DICT node.
      *
      * @param node the node of type #PLIST_DICT
      * @param iter iterator of the dictionary
-     * @param key a location to store the key, or NULL.
-     * @param val a location to store the value, or NULL.
+     * @param key a location to store the key, or NULL. The caller is responsible
+     *		for freeing the the returned string.
+     * @param val a location to store the value, or NULL. The caller should *not*
+     *		free the returned value.
      */
-    PLIST_API void plist_dict_next_item(plist_t node, plist_dict_iter iter, char **key, plist_t *val);
+    void plist_dict_next_item(plist_t node, plist_dict_iter iter, char **key, plist_t *val);
 
     /**
      * Get key associated to an item. Item must be member of a dictionary
      *
      * @param node the node
-     * @param key a location to store the key.
+     * @param key a location to store the key. The caller is responsible for freeing the returned string.
      */
-    PLIST_API void plist_dict_get_item_key(plist_t node, char **key);
+    void plist_dict_get_item_key(plist_t node, char **key);
 
     /**
      * Get the nth item in a #PLIST_DICT node.
      *
      * @param node the node of type #PLIST_DICT
      * @param key the identifier of the item to get.
-     * @return the item or NULL if node is not of type #PLIST_DICT
+     * @return the item or NULL if node is not of type #PLIST_DICT. The caller should not free
+     *		the returned node.
      */
-    PLIST_API plist_t plist_dict_get_item(plist_t node, const char* key);
+    plist_t plist_dict_get_item(plist_t node, const char* key);
 
     /**
      * Set item identified by key in a #PLIST_DICT node.
@@ -325,7 +343,7 @@ extern "C"
      * @param item the new item associated to key
      * @param key the identifier of the item to set.
      */
-    PLIST_API void plist_dict_set_item(plist_t node, const char* key, plist_t item);
+    void plist_dict_set_item(plist_t node, const char* key, plist_t item);
 
     /**
      * Insert a new item into a #PLIST_DICT node.
@@ -336,7 +354,8 @@ extern "C"
      * @param item the new item to insert
      * @param key The identifier of the item to insert.
      */
-    DEPRECATED("use plist_dict_set_item instead") PLIST_API void plist_dict_insert_item(plist_t node, const char* key, plist_t item);
+    PLIST_WARN_DEPRECATED("use plist_dict_set_item instead")
+    void plist_dict_insert_item(plist_t node, const char* key, plist_t item);
 
     /**
      * Remove an existing position in a #PLIST_DICT node.
@@ -345,7 +364,7 @@ extern "C"
      * @param node the node of type #PLIST_DICT
      * @param key The identifier of the item to remove. Assert if identifier is not present.
      */
-    PLIST_API void plist_dict_remove_item(plist_t node, const char* key);
+    void plist_dict_remove_item(plist_t node, const char* key);
 
     /**
      * Merge a dictionary into another. This will add all key/value pairs
@@ -355,7 +374,7 @@ extern "C"
      * @param target pointer to an existing node of type #PLIST_DICT
      * @param source node of type #PLIST_DICT that should be merged into target
      */
-    PLIST_API void plist_dict_merge(plist_t *target, plist_t source);
+    void plist_dict_merge(plist_t *target, plist_t source);
 
 
     /********************************************
@@ -369,7 +388,7 @@ extern "C"
      *
      * @param node the parent (NULL if node is root)
      */
-    PLIST_API plist_t plist_get_parent(plist_t node);
+    plist_t plist_get_parent(plist_t node);
 
     /**
      * Get the #plist_type of a node.
@@ -377,7 +396,7 @@ extern "C"
      * @param node the node
      * @return the type of the node
      */
-    PLIST_API plist_type plist_get_node_type(plist_t node);
+    plist_type plist_get_node_type(plist_t node);
 
     /**
      * Get the value of a #PLIST_KEY node.
@@ -387,7 +406,7 @@ extern "C"
      * @param val a pointer to a C-string. This function allocates the memory,
      *            caller is responsible for freeing it.
      */
-    PLIST_API void plist_get_key_val(plist_t node, char **val);
+    void plist_get_key_val(plist_t node, char **val);
 
     /**
      * Get the value of a #PLIST_STRING node.
@@ -397,7 +416,7 @@ extern "C"
      * @param val a pointer to a C-string. This function allocates the memory,
      *            caller is responsible for freeing it. Data is UTF-8 encoded.
      */
-    PLIST_API void plist_get_string_val(plist_t node, char **val);
+    void plist_get_string_val(plist_t node, char **val);
 
     /**
      * Get the value of a #PLIST_BOOLEAN node.
@@ -406,7 +425,7 @@ extern "C"
      * @param node the node
      * @param val a pointer to a uint8_t variable.
      */
-    PLIST_API void plist_get_bool_val(plist_t node, uint8_t * val);
+    void plist_get_bool_val(plist_t node, uint8_t * val);
 
     /**
      * Get the value of a #PLIST_UINT node.
@@ -415,7 +434,7 @@ extern "C"
      * @param node the node
      * @param val a pointer to a uint64_t variable.
      */
-    PLIST_API void plist_get_uint_val(plist_t node, uint64_t * val);
+    void plist_get_uint_val(plist_t node, uint64_t * val);
 
     /**
      * Get the value of a #PLIST_REAL node.
@@ -424,7 +443,7 @@ extern "C"
      * @param node the node
      * @param val a pointer to a double variable.
      */
-    PLIST_API void plist_get_real_val(plist_t node, double *val);
+    void plist_get_real_val(plist_t node, double *val);
 
     /**
      * Get the value of a #PLIST_DATA node.
@@ -435,7 +454,7 @@ extern "C"
      *            caller is responsible for freeing it.
      * @param length the length of the buffer
      */
-    PLIST_API void plist_get_data_val(plist_t node, char **val, uint64_t * length);
+    void plist_get_data_val(plist_t node, char **val, uint64_t * length);
 
     /**
      * Get the value of a #PLIST_DATE node.
@@ -445,7 +464,7 @@ extern "C"
      * @param sec a pointer to an int32_t variable. Represents the number of seconds since 01/01/2001.
      * @param usec a pointer to an int32_t variable. Represents the number of microseconds
      */
-    PLIST_API void plist_get_date_val(plist_t node, int32_t * sec, int32_t * usec);
+    void plist_get_date_val(plist_t node, int32_t * sec, int32_t * usec);
 
     /**
      * Get the value of a #PLIST_UID node.
@@ -454,7 +473,7 @@ extern "C"
      * @param node the node
      * @param val a pointer to a uint64_t variable.
      */
-    PLIST_API void plist_get_uid_val(plist_t node, uint64_t * val);
+    void plist_get_uid_val(plist_t node, uint64_t * val);
 
 
     /********************************************
@@ -463,14 +482,6 @@ extern "C"
      *                                          *
      ********************************************/
 
-    /**
-     * Forces type of node. Changing type of structured nodes is only allowed if node is empty.
-     * Reset value of node;
-     * @param node the node
-     * @param type the key value
-     */
-    PLIST_API void plist_set_type(plist_t node, plist_type type);
-
     /**
      * Set the value of a node.
      * Forces type of node to #PLIST_KEY
@@ -478,16 +489,17 @@ extern "C"
      * @param node the node
      * @param val the key value
      */
-    PLIST_API void plist_set_key_val(plist_t node, const char *val);
+    void plist_set_key_val(plist_t node, const char *val);
 
     /**
      * Set the value of a node.
      * Forces type of node to #PLIST_STRING
      *
      * @param node the node
-     * @param val the string value
+     * @param val the string value. The string is copied when set and will be
+     *		freed by the node.
      */
-    PLIST_API void plist_set_string_val(plist_t node, const char *val);
+    void plist_set_string_val(plist_t node, const char *val);
 
     /**
      * Set the value of a node.
@@ -496,7 +508,7 @@ extern "C"
      * @param node the node
      * @param val the boolean value
      */
-    PLIST_API void plist_set_bool_val(plist_t node, uint8_t val);
+    void plist_set_bool_val(plist_t node, uint8_t val);
 
     /**
      * Set the value of a node.
@@ -505,7 +517,7 @@ extern "C"
      * @param node the node
      * @param val the unsigned integer value
      */
-    PLIST_API void plist_set_uint_val(plist_t node, uint64_t val);
+    void plist_set_uint_val(plist_t node, uint64_t val);
 
     /**
      * Set the value of a node.
@@ -514,17 +526,18 @@ extern "C"
      * @param node the node
      * @param val the real value
      */
-    PLIST_API void plist_set_real_val(plist_t node, double val);
+    void plist_set_real_val(plist_t node, double val);
 
     /**
      * Set the value of a node.
      * Forces type of node to #PLIST_DATA
      *
      * @param node the node
-     * @param val the binary buffer
+     * @param val the binary buffer. The buffer is copied when set and will
+     *		be freed by the node.
      * @param length the length of the buffer
      */
-    PLIST_API void plist_set_data_val(plist_t node, const char *val, uint64_t length);
+    void plist_set_data_val(plist_t node, const char *val, uint64_t length);
 
     /**
      * Set the value of a node.
@@ -534,7 +547,7 @@ extern "C"
      * @param sec the number of seconds since 01/01/2001
      * @param usec the number of microseconds
      */
-    PLIST_API void plist_set_date_val(plist_t node, int32_t sec, int32_t usec);
+    void plist_set_date_val(plist_t node, int32_t sec, int32_t usec);
 
     /**
      * Set the value of a node.
@@ -543,7 +556,7 @@ extern "C"
      * @param node the node
      * @param val the unsigned integer value
      */
-    PLIST_API void plist_set_uid_val(plist_t node, uint64_t val);
+    void plist_set_uid_val(plist_t node, uint64_t val);
 
 
     /********************************************
@@ -560,7 +573,7 @@ extern "C"
      *            caller is responsible for freeing it. Data is UTF-8 encoded.
      * @param length a pointer to an uint32_t variable. Represents the length of the allocated buffer.
      */
-    PLIST_API void plist_to_xml(plist_t plist, char **plist_xml, uint32_t * length);
+    void plist_to_xml(plist_t plist, char **plist_xml, uint32_t * length);
 
     /**
      * Export the #plist_t structure to binary format.
@@ -570,7 +583,7 @@ extern "C"
      *            caller is responsible for freeing it.
      * @param length a pointer to an uint32_t variable. Represents the length of the allocated buffer.
      */
-    PLIST_API void plist_to_bin(plist_t plist, char **plist_bin, uint32_t * length);
+    void plist_to_bin(plist_t plist, char **plist_bin, uint32_t * length);
 
     /**
      * Import the #plist_t structure from XML format.
@@ -579,7 +592,7 @@ extern "C"
      * @param length length of the buffer to read.
      * @param plist a pointer to the imported plist.
      */
-    PLIST_API void plist_from_xml(const char *plist_xml, uint32_t length, plist_t * plist);
+    void plist_from_xml(const char *plist_xml, uint32_t length, plist_t * plist);
 
     /**
      * Import the #plist_t structure from binary format.
@@ -588,7 +601,7 @@ extern "C"
      * @param length length of the buffer to read.
      * @param plist a pointer to the imported plist.
      */
-    PLIST_API void plist_from_bin(const char *plist_bin, uint32_t length, plist_t * plist);
+    void plist_from_bin(const char *plist_bin, uint32_t length, plist_t * plist);
 
 
     /********************************************
@@ -606,7 +619,7 @@ extern "C"
      * @param length length of the path to access
      * @return the value to access.
      */
-    PLIST_API plist_t plist_access_path(plist_t plist, uint32_t length, ...);
+    plist_t plist_access_path(plist_t plist, uint32_t length, ...);
 
     /**
      * Variadic version of #plist_access_path.
@@ -616,7 +629,7 @@ extern "C"
      * @param v list of array's index and dic'st key
      * @return the value to access.
      */
-    PLIST_API plist_t plist_access_pathv(plist_t plist, uint32_t length, va_list v);
+    plist_t plist_access_pathv(plist_t plist, uint32_t length, va_list v);
 
     /**
      * Compare two node values
@@ -625,7 +638,7 @@ extern "C"
      * @param node_r rigth node to compare
      * @return TRUE is type and value match, FALSE otherwise.
      */
-    PLIST_API char plist_compare_node_value(plist_t node_l, plist_t node_r);
+    char plist_compare_node_value(plist_t node_l, plist_t node_r);
 
     /*@}*/
 
