@@ -19,6 +19,9 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -73,7 +76,7 @@ enum
 
 static void float_byte_convert(uint8_t * address, size_t size)
 {
-#if (PLIST_BYTE_ORDER == PLIST_LITTLE_ENDIAN \
+#if (defined(__LITTLE_ENDIAN__) \
 		&& !defined(__FLOAT_WORD_ORDER__)) \
 	|| (defined(__FLOAT_WORD_ORDER__) \
 		&& __FLOAT_WORD_ORDER__ == __ORDER_LITTLE_ENDIAN__)
@@ -110,7 +113,7 @@ union plist_uint_ptr
 
 static void byte_convert(uint8_t * address, size_t size)
 {
-#if PLIST_BYTE_ORDER == PLIST_LITTLE_ENDIAN
+#ifdef __LITTLE_ENDIAN__
     uint8_t i = 0, j = 0;
     uint8_t tmp = 0;
 
@@ -138,7 +141,7 @@ static uint32_t uint24_from_be(union plist_uint_ptr buf)
 }
 
 #ifndef be16toh
-#if PLIST_BYTE_ORDER == PLIST_BIG_ENDIAN
+#ifdef __BIG_ENDIAN__
 #define be16toh(x) (x)
 #else
 #define be16toh(x) ((((x) & 0xFF00) >> 8) | (((x) & 0x00FF) << 8))
@@ -146,7 +149,7 @@ static uint32_t uint24_from_be(union plist_uint_ptr buf)
 #endif
 
 #ifndef be32toh
-#if PLIST_BYTE_ORDER == PLIST_BIG_ENDIAN
+#ifdef __BIG_ENDIAN__
 #define be32toh(x) (x)
 #else
 #define be32toh(x) ((((x) & 0xFF000000) >> 24) \
@@ -157,7 +160,7 @@ static uint32_t uint24_from_be(union plist_uint_ptr buf)
 #endif
 
 #ifndef be64toh
-#if PLIST_BYTE_ORDER == PLIST_BIG_ENDIAN
+#ifdef __BIG_ENDIAN__
 #define be64toh(x) (x)
 #else
 #define be64toh(x) ((((x) & 0xFF00000000000000ull) >> 56) \
@@ -212,6 +215,12 @@ static plist_t parse_uint_node(char *bnode, uint8_t size, char **next_object)
     case sizeof(uint64_t):
         memcpy(&data->intval, bnode, size);
         data->intval = UINT_TO_HOST(&data->intval, size);
+        data->length = sizeof(uint64_t);
+        break;
+    case 16:
+        memcpy(&data->intval, bnode+8, sizeof(uint64_t));
+        data->intval = UINT_TO_HOST(&data->intval, sizeof(uint64_t));
+        data->length = size;
         break;
     default:
         free(data);
@@ -220,7 +229,6 @@ static plist_t parse_uint_node(char *bnode, uint8_t size, char **next_object)
 
     *next_object = bnode + size;
     data->type = PLIST_UINT;
-    data->length = sizeof(uint64_t);
 
     return node_create(NULL, data);
 }
@@ -246,6 +254,7 @@ static plist_t parse_real_node(char *bnode, uint8_t size)
         data->realval = *(double *) buf;
         break;
     default:
+        free(buf);
         free(data);
         return NULL;
     }
@@ -593,7 +602,7 @@ static void* copy_plist_data(const void* src)
     return dstdata;
 }
 
-void plist_from_bin(const char *plist_bin, uint32_t length, plist_t * plist)
+PLIST_API void plist_from_bin(const char *plist_bin, uint32_t length, plist_t * plist)
 {
     char *trailer = NULL;
 
@@ -664,29 +673,51 @@ void plist_from_bin(const char *plist_bin, uint32_t length, plist_t * plist)
         case PLIST_DICT:
             for (j = 0; j < data->length; j++)
             {
+                node_t* n = NULL;
                 str_i = j * dict_param_size;
                 str_j = (j + data->length) * dict_param_size;
 
                 index1 = UINT_TO_HOST(data->buff + str_i, dict_param_size);
                 index2 = UINT_TO_HOST(data->buff + str_j, dict_param_size);
 
-                //first one is actually a key
-                plist_get_data(nodeslist[index1])->type = PLIST_KEY;
-
+                // process key node
                 if (index1 < num_objects)
                 {
-                    if (NODE_IS_ROOT(nodeslist[index1]))
-                        node_attach(nodeslist[i], nodeslist[index1]);
-                    else
-                        node_attach(nodeslist[i], node_copy_deep(nodeslist[index1], copy_plist_data));
+                    // is node already attached?
+                    if (NODE_IS_ROOT(nodeslist[index1])) {
+                        // use original
+                        n = nodeslist[index1];
+                    } else {
+                        // use a copy, because this node is already attached elsewhere
+                        n = node_copy_deep(nodeslist[index1], copy_plist_data);
+                    }
+
+                    // enforce key type
+                    plist_get_data(n)->type = PLIST_KEY;
+
+                    // attach key node
+                    node_attach(nodeslist[i], n);
                 }
 
+                // process value node
                 if (index2 < num_objects)
                 {
-                    if (NODE_IS_ROOT(nodeslist[index2]))
-                        node_attach(nodeslist[i], nodeslist[index2]);
-                    else
-                        node_attach(nodeslist[i], node_copy_deep(nodeslist[index2], copy_plist_data));
+                    // is node already attached?
+                    if (NODE_IS_ROOT(nodeslist[index2])) {
+                        // use original
+                        n = nodeslist[index2];
+                    } else {
+                        // use a copy, because this node is already attached elsewhere
+                        n = node_copy_deep(nodeslist[index2], copy_plist_data);
+
+                        // ensure key type is never used for values, especially if we copy a key node
+                        if (plist_get_data(n)->type == PLIST_KEY) {
+                            plist_get_data(n)->type = PLIST_STRING;
+                        }
+                    }
+
+                    // attach value node
+                    node_attach(nodeslist[i], n);
                 }
             }
 
@@ -715,6 +746,15 @@ void plist_from_bin(const char *plist_bin, uint32_t length, plist_t * plist)
     }
 
     *plist = nodeslist[root_object];
+
+    // free unreferenced nodes that would otherwise leak memory
+    for (i = 0; i < num_objects; i++) {
+        if (i == root_object) continue;
+        node_t* node = (node_t*)nodeslist[i];
+        if (node && NODE_IS_ROOT(node)) {
+            plist_free(node);
+        }
+    }
     free(nodeslist);
 }
 
@@ -812,7 +852,7 @@ static void write_int(bytearray_t * bplist, uint64_t val)
     if (size == 3)
         size++;
 
-#if PLIST_BYTE_ORDER == PLIST_BIG_ENDIAN
+#ifdef __BIG_ENDIAN__
     val = val << ((sizeof(uint64_t) - size) * 8);
 #endif
 
@@ -824,6 +864,20 @@ static void write_int(bytearray_t * bplist, uint64_t val)
     free(buff);
 }
 
+static void write_uint(bytearray_t * bplist, uint64_t val)
+{
+    uint64_t size = 16;
+    uint8_t *buff = NULL;
+
+    buff = (uint8_t *) malloc(sizeof(uint8_t) + size);
+    buff[0] = BPLIST_UINT | 4;
+    memset(buff + 1, '\0', 8);
+    memcpy(buff + 9, &val, 8);
+    byte_convert(buff + 9, 8);
+    byte_array_append(bplist, buff, sizeof(uint8_t) + size);
+    free(buff);
+}
+
 static void write_real(bytearray_t * bplist, double val)
 {
     uint64_t size = get_real_bytes(val);	//cheat to know used space
@@ -921,7 +975,7 @@ static void write_array(bytearray_t * bplist, node_t* node, hashtable_t* ref_tab
     for (i = 0, cur = node_first_child(node); cur && i < size; cur = node_next_sibling(cur), i++)
     {
         idx = *(uint64_t *) (hash_table_lookup(ref_table, cur));
-#if PLIST_BYTE_ORDER == PLIST_BIG_ENDIAN
+#ifdef __BIG_ENDIAN__
 	idx = idx << ((sizeof(uint64_t) - dict_param_size) * 8);
 #endif
         memcpy(buff + i * dict_param_size, &idx, dict_param_size);
@@ -958,14 +1012,14 @@ static void write_dict(bytearray_t * bplist, node_t* node, hashtable_t* ref_tabl
     for (i = 0, cur = node_first_child(node); cur && i < size; cur = node_next_sibling(node_next_sibling(cur)), i++)
     {
         idx1 = *(uint64_t *) (hash_table_lookup(ref_table, cur));
-#if PLIST_BYTE_ORDER == PLIST_BIG_ENDIAN
+#ifdef __BIG_ENDIAN__
 	idx1 = idx1 << ((sizeof(uint64_t) - dict_param_size) * 8);
 #endif
         memcpy(buff + i * dict_param_size, &idx1, dict_param_size);
         byte_convert(buff + i * dict_param_size, dict_param_size);
 
         idx2 = *(uint64_t *) (hash_table_lookup(ref_table, cur->next));
-#if PLIST_BYTE_ORDER == PLIST_BIG_ENDIAN
+#ifdef __BIG_ENDIAN__
 	idx2 = idx2 << ((sizeof(uint64_t) - dict_param_size) * 8);
 #endif
         memcpy(buff + (i + size) * dict_param_size, &idx2, dict_param_size);
@@ -986,7 +1040,7 @@ static void write_uid(bytearray_t * bplist, uint64_t val)
     if (size == 3)
         size++;
 
-#if PLIST_BYTE_ORDER == PLIST_BIG_ENDIAN
+#ifdef __BIG_ENDIAN__
     val = val << ((sizeof(uint64_t) - size) * 8);
 #endif
 
@@ -1066,7 +1120,7 @@ static uint16_t *plist_utf8_to_utf16(char *unistr, long size, long *items_read,
 
 }
 
-void plist_to_bin(plist_t plist, char **plist_bin, uint32_t * length)
+PLIST_API void plist_to_bin(plist_t plist, char **plist_bin, uint32_t * length)
 {
     ptrarray_t* objects = NULL;
     hashtable_t* ref_table = NULL;
@@ -1087,6 +1141,8 @@ void plist_to_bin(plist_t plist, char **plist_bin, uint32_t * length)
     long items_read = 0;
     long items_written = 0;
     uint16_t *unicodestr = NULL;
+    uint64_t objects_len = 0;
+    uint64_t buff_len = 0;
 
     //check for valid input
     if (!plist || !plist_bin || *plist_bin || !length)
@@ -1104,7 +1160,8 @@ void plist_to_bin(plist_t plist, char **plist_bin, uint32_t * length)
 
     //now stream to output buffer
     offset_size = 0;			//unknown yet
-    dict_param_size = get_needed_bytes(objects->len);
+    objects_len = objects->len;
+    dict_param_size = get_needed_bytes(objects_len);
     num_objects = objects->len;
     root_object = 0;			//root is first in list
     offset_table_index = 0;		//unknown yet
@@ -1134,7 +1191,11 @@ void plist_to_bin(plist_t plist, char **plist_bin, uint32_t * length)
             break;
 
         case PLIST_UINT:
-            write_int(bplist_buff, data->intval);
+            if (data->length == 16) {
+                write_uint(bplist_buff, data->intval);
+            } else {
+                write_int(bplist_buff, data->intval);
+            }
             break;
 
         case PLIST_REAL:
@@ -1179,13 +1240,14 @@ void plist_to_bin(plist_t plist, char **plist_bin, uint32_t * length)
     hash_table_destroy(ref_table);
 
     //write offsets
-    offset_size = get_needed_bytes(bplist_buff->len);
+    buff_len = bplist_buff->len;
+    offset_size = get_needed_bytes(buff_len);
     offset_table_index = bplist_buff->len;
     for (i = 0; i < num_objects; i++)
     {
         uint8_t *offsetbuff = (uint8_t *) malloc(offset_size);
 
-#if PLIST_BYTE_ORDER == PLIST_BIG_ENDIAN
+#ifdef __BIG_ENDIAN__
 	offsets[i] = offsets[i] << ((sizeof(uint64_t) - offset_size) * 8);
 #endif
 
