@@ -70,7 +70,7 @@ static void plist_free_data(plist_data_t data)
 static int plist_free_node(node_t* node)
 {
     plist_data_t data = NULL;
-    int index = node_detach(node->parent, node);
+    int node_index = node_detach(node->parent, node);
     data = plist_get_data(node);
     plist_free_data(data);
     node->data = NULL;
@@ -84,17 +84,17 @@ static int plist_free_node(node_t* node)
 
     node_destroy(node);
 
-    return index;
+    return node_index;
 }
 
-plist_t plist_new_dict(void)
+PLIST_API plist_t plist_new_dict(void)
 {
     plist_data_t data = plist_new_plist_data();
     data->type = PLIST_DICT;
     return plist_new_node(data);
 }
 
-plist_t plist_new_array(void)
+PLIST_API plist_t plist_new_array(void)
 {
     plist_data_t data = plist_new_plist_data();
     data->type = PLIST_ARRAY;
@@ -111,7 +111,7 @@ static plist_t plist_new_key(const char *val)
     return plist_new_node(data);
 }
 
-plist_t plist_new_string(const char *val)
+PLIST_API plist_t plist_new_string(const char *val)
 {
     plist_data_t data = plist_new_plist_data();
     data->type = PLIST_STRING;
@@ -120,7 +120,7 @@ plist_t plist_new_string(const char *val)
     return plist_new_node(data);
 }
 
-plist_t plist_new_bool(uint8_t val)
+PLIST_API plist_t plist_new_bool(uint8_t val)
 {
     plist_data_t data = plist_new_plist_data();
     data->type = PLIST_BOOLEAN;
@@ -129,7 +129,7 @@ plist_t plist_new_bool(uint8_t val)
     return plist_new_node(data);
 }
 
-plist_t plist_new_uint(uint64_t val)
+PLIST_API plist_t plist_new_uint(uint64_t val)
 {
     plist_data_t data = plist_new_plist_data();
     data->type = PLIST_UINT;
@@ -138,7 +138,7 @@ plist_t plist_new_uint(uint64_t val)
     return plist_new_node(data);
 }
 
-plist_t plist_new_uid(uint64_t val)
+PLIST_API plist_t plist_new_uid(uint64_t val)
 {
     plist_data_t data = plist_new_plist_data();
     data->type = PLIST_UID;
@@ -147,7 +147,7 @@ plist_t plist_new_uid(uint64_t val)
     return plist_new_node(data);
 }
 
-plist_t plist_new_real(double val)
+PLIST_API plist_t plist_new_real(double val)
 {
     plist_data_t data = plist_new_plist_data();
     data->type = PLIST_REAL;
@@ -156,7 +156,7 @@ plist_t plist_new_real(double val)
     return plist_new_node(data);
 }
 
-plist_t plist_new_data(const char *val, uint64_t length)
+PLIST_API plist_t plist_new_data(const char *val, uint64_t length)
 {
     plist_data_t data = plist_new_plist_data();
     data->type = PLIST_DATA;
@@ -166,7 +166,7 @@ plist_t plist_new_data(const char *val, uint64_t length)
     return plist_new_node(data);
 }
 
-plist_t plist_new_date(int32_t sec, int32_t usec)
+PLIST_API plist_t plist_new_date(int32_t sec, int32_t usec)
 {
     plist_data_t data = plist_new_plist_data();
     data->type = PLIST_DATE;
@@ -176,7 +176,7 @@ plist_t plist_new_date(int32_t sec, int32_t usec)
     return plist_new_node(data);
 }
 
-void plist_free(plist_t plist)
+PLIST_API void plist_free(plist_t plist)
 {
     if (plist)
     {
@@ -231,14 +231,14 @@ static void plist_copy_node(node_t *node, void *parent_node_ptr)
     node_iterator_destroy(ni);
 }
 
-plist_t plist_copy(plist_t node)
+PLIST_API plist_t plist_copy(plist_t node)
 {
     plist_t copied = NULL;
     plist_copy_node(node, &copied);
     return copied;
 }
 
-uint32_t plist_array_get_size(plist_t node)
+PLIST_API uint32_t plist_array_get_size(plist_t node)
 {
     uint32_t ret = 0;
     if (node && PLIST_ARRAY == plist_get_node_type(node))
@@ -248,7 +248,7 @@ uint32_t plist_array_get_size(plist_t node)
     return ret;
 }
 
-plist_t plist_array_get_item(plist_t node, uint32_t n)
+PLIST_API plist_t plist_array_get_item(plist_t node, uint32_t n)
 {
     plist_t ret = NULL;
     if (node && PLIST_ARRAY == plist_get_node_type(node))
@@ -258,7 +258,7 @@ plist_t plist_array_get_item(plist_t node, uint32_t n)
     return ret;
 }
 
-uint32_t plist_array_get_item_index(plist_t node)
+PLIST_API uint32_t plist_array_get_item_index(plist_t node)
 {
     plist_t father = plist_get_parent(node);
     if (PLIST_ARRAY == plist_get_node_type(father))
@@ -268,7 +268,7 @@ uint32_t plist_array_get_item_index(plist_t node)
     return 0;
 }
 
-void plist_array_set_item(plist_t node, plist_t item, uint32_t n)
+PLIST_API void plist_array_set_item(plist_t node, plist_t item, uint32_t n)
 {
     if (node && PLIST_ARRAY == plist_get_node_type(node))
     {
@@ -286,7 +286,7 @@ void plist_array_set_item(plist_t node, plist_t item, uint32_t n)
     return;
 }
 
-void plist_array_append_item(plist_t node, plist_t item)
+PLIST_API void plist_array_append_item(plist_t node, plist_t item)
 {
     if (node && PLIST_ARRAY == plist_get_node_type(node))
     {
@@ -295,7 +295,7 @@ void plist_array_append_item(plist_t node, plist_t item)
     return;
 }
 
-void plist_array_insert_item(plist_t node, plist_t item, uint32_t n)
+PLIST_API void plist_array_insert_item(plist_t node, plist_t item, uint32_t n)
 {
     if (node && PLIST_ARRAY == plist_get_node_type(node))
     {
@@ -304,7 +304,7 @@ void plist_array_insert_item(plist_t node, plist_t item, uint32_t n)
     return;
 }
 
-void plist_array_remove_item(plist_t node, uint32_t n)
+PLIST_API void plist_array_remove_item(plist_t node, uint32_t n)
 {
     if (node && PLIST_ARRAY == plist_get_node_type(node))
     {
@@ -317,7 +317,7 @@ void plist_array_remove_item(plist_t node, uint32_t n)
     return;
 }
 
-uint32_t plist_dict_get_size(plist_t node)
+PLIST_API uint32_t plist_dict_get_size(plist_t node)
 {
     uint32_t ret = 0;
     if (node && PLIST_DICT == plist_get_node_type(node))
@@ -327,7 +327,7 @@ uint32_t plist_dict_get_size(plist_t node)
     return ret;
 }
 
-void plist_dict_new_iter(plist_t node, plist_dict_iter *iter)
+PLIST_API void plist_dict_new_iter(plist_t node, plist_dict_iter *iter)
 {
     if (iter && *iter == NULL)
     {
@@ -337,7 +337,7 @@ void plist_dict_new_iter(plist_t node, plist_dict_iter *iter)
     return;
 }
 
-void plist_dict_next_item(plist_t node, plist_dict_iter iter, char **key, plist_t *val)
+PLIST_API void plist_dict_next_item(plist_t node, plist_dict_iter iter, char **key, plist_t *val)
 {
     uint32_t* iter_int = (uint32_t*) iter;
 
@@ -368,7 +368,7 @@ void plist_dict_next_item(plist_t node, plist_dict_iter iter, char **key, plist_
     return;
 }
 
-void plist_dict_get_item_key(plist_t node, char **key)
+PLIST_API void plist_dict_get_item_key(plist_t node, char **key)
 {
     plist_t father = plist_get_parent(node);
     if (PLIST_DICT == plist_get_node_type(father))
@@ -377,7 +377,7 @@ void plist_dict_get_item_key(plist_t node, char **key)
     }
 }
 
-plist_t plist_dict_get_item(plist_t node, const char* key)
+PLIST_API plist_t plist_dict_get_item(plist_t node, const char* key)
 {
     plist_t ret = NULL;
 
@@ -403,7 +403,7 @@ plist_t plist_dict_get_item(plist_t node, const char* key)
     return ret;
 }
 
-void plist_dict_set_item(plist_t node, const char* key, plist_t item)
+PLIST_API void plist_dict_set_item(plist_t node, const char* key, plist_t item)
 {
     if (node && PLIST_DICT == plist_get_node_type(node)) {
         node_t* old_item = plist_dict_get_item(node, key);
@@ -422,12 +422,12 @@ void plist_dict_set_item(plist_t node, const char* key, plist_t item)
     return;
 }
 
-void plist_dict_insert_item(plist_t node, const char* key, plist_t item)
+PLIST_API void plist_dict_insert_item(plist_t node, const char* key, plist_t item)
 {
     plist_dict_set_item(node, key, item);
 }
 
-void plist_dict_remove_item(plist_t node, const char* key)
+PLIST_API void plist_dict_remove_item(plist_t node, const char* key)
 {
     if (node && PLIST_DICT == plist_get_node_type(node))
     {
@@ -442,7 +442,7 @@ void plist_dict_remove_item(plist_t node, const char* key)
     return;
 }
 
-void plist_dict_merge(plist_t *target, plist_t source)
+PLIST_API void plist_dict_merge(plist_t *target, plist_t source)
 {
 	if (!target || !*target || (plist_get_node_type(*target) != PLIST_DICT) || !source || (plist_get_node_type(source) != PLIST_DICT))
 		return;
@@ -459,9 +459,6 @@ void plist_dict_merge(plist_t *target, plist_t source)
 		if (!key)
 			break;
 
-		if (plist_dict_get_item(*target, key) != NULL)
-			plist_dict_remove_item(*target, key);
-
 		plist_dict_set_item(*target, key, plist_copy(subnode));
 		free(key);
 		key = NULL;
@@ -469,7 +466,7 @@ void plist_dict_merge(plist_t *target, plist_t source)
 	free(it);	
 }
 
-plist_t plist_access_pathv(plist_t plist, uint32_t length, va_list v)
+PLIST_API plist_t plist_access_pathv(plist_t plist, uint32_t length, va_list v)
 {
     plist_t current = plist;
     plist_type type = PLIST_NONE;
@@ -493,7 +490,7 @@ plist_t plist_access_pathv(plist_t plist, uint32_t length, va_list v)
     return current;
 }
 
-plist_t plist_access_path(plist_t plist, uint32_t length, ...)
+PLIST_API plist_t plist_access_path(plist_t plist, uint32_t length, ...)
 {
     plist_t ret = NULL;
     va_list v;
@@ -548,12 +545,12 @@ static void plist_get_type_and_value(plist_t node, plist_type * type, void *valu
     }
 }
 
-plist_t plist_get_parent(plist_t node)
+PLIST_API plist_t plist_get_parent(plist_t node)
 {
     return node ? (plist_t) ((node_t*) node)->parent : NULL;
 }
 
-plist_type plist_get_node_type(plist_t node)
+PLIST_API plist_type plist_get_node_type(plist_t node)
 {
     if (node)
     {
@@ -564,7 +561,7 @@ plist_type plist_get_node_type(plist_t node)
     return PLIST_NONE;
 }
 
-void plist_get_key_val(plist_t node, char **val)
+PLIST_API void plist_get_key_val(plist_t node, char **val)
 {
     plist_type type = plist_get_node_type(node);
     uint64_t length = 0;
@@ -573,7 +570,7 @@ void plist_get_key_val(plist_t node, char **val)
     assert(length == strlen(*val));
 }
 
-void plist_get_string_val(plist_t node, char **val)
+PLIST_API void plist_get_string_val(plist_t node, char **val)
 {
     plist_type type = plist_get_node_type(node);
     uint64_t length = 0;
@@ -582,7 +579,7 @@ void plist_get_string_val(plist_t node, char **val)
     assert(length == strlen(*val));
 }
 
-void plist_get_bool_val(plist_t node, uint8_t * val)
+PLIST_API void plist_get_bool_val(plist_t node, uint8_t * val)
 {
     plist_type type = plist_get_node_type(node);
     uint64_t length = 0;
@@ -591,7 +588,7 @@ void plist_get_bool_val(plist_t node, uint8_t * val)
     assert(length == sizeof(uint8_t));
 }
 
-void plist_get_uint_val(plist_t node, uint64_t * val)
+PLIST_API void plist_get_uint_val(plist_t node, uint64_t * val)
 {
     plist_type type = plist_get_node_type(node);
     uint64_t length = 0;
@@ -600,7 +597,7 @@ void plist_get_uint_val(plist_t node, uint64_t * val)
     assert(length == sizeof(uint64_t));
 }
 
-void plist_get_uid_val(plist_t node, uint64_t * val)
+PLIST_API void plist_get_uid_val(plist_t node, uint64_t * val)
 {
     plist_type type = plist_get_node_type(node);
     uint64_t length = 0;
@@ -609,7 +606,7 @@ void plist_get_uid_val(plist_t node, uint64_t * val)
     assert(length == sizeof(uint64_t));
 }
 
-void plist_get_real_val(plist_t node, double *val)
+PLIST_API void plist_get_real_val(plist_t node, double *val)
 {
     plist_type type = plist_get_node_type(node);
     uint64_t length = 0;
@@ -618,14 +615,14 @@ void plist_get_real_val(plist_t node, double *val)
     assert(length == sizeof(double));
 }
 
-void plist_get_data_val(plist_t node, char **val, uint64_t * length)
+PLIST_API void plist_get_data_val(plist_t node, char **val, uint64_t * length)
 {
     plist_type type = plist_get_node_type(node);
     if (PLIST_DATA == type)
         plist_get_type_and_value(node, &type, (void *) val, length);
 }
 
-void plist_get_date_val(plist_t node, int32_t * sec, int32_t * usec)
+PLIST_API void plist_get_date_val(plist_t node, int32_t * sec, int32_t * usec)
 {
     plist_type type = plist_get_node_type(node);
     uint64_t length = 0;
@@ -698,7 +695,7 @@ int plist_data_compare(const void *a, const void *b)
     return FALSE;
 }
 
-char plist_compare_node_value(plist_t node_l, plist_t node_r)
+PLIST_API char plist_compare_node_value(plist_t node_l, plist_t node_r)
 {
     return plist_data_compare(node_l, node_r);
 }
@@ -760,72 +757,42 @@ static void plist_set_element_val(plist_t node, plist_type type, const void *val
     }
 }
 
-void plist_set_type(plist_t node, plist_type type)
-{
-    if ( node_n_children(node) == 0 )
-    {
-        plist_data_t data = plist_get_data(node);
-        plist_free_data( data );
-        data = plist_new_plist_data();
-        data->type = type;
-        switch (type)
-        {
-        case PLIST_BOOLEAN:
-            data->length = sizeof(uint8_t);
-            break;
-        case PLIST_UINT:
-        case PLIST_UID:
-            data->length = sizeof(uint64_t);
-            break;
-        case PLIST_REAL:
-            data->length = sizeof(double);
-            break;
-        case PLIST_DATE:
-            data->length = sizeof(struct timeval);
-            break;
-        default:
-            data->length = 0;
-            break;
-        }
-    }
-}
-
-void plist_set_key_val(plist_t node, const char *val)
+PLIST_API void plist_set_key_val(plist_t node, const char *val)
 {
     plist_set_element_val(node, PLIST_KEY, val, strlen(val));
 }
 
-void plist_set_string_val(plist_t node, const char *val)
+PLIST_API void plist_set_string_val(plist_t node, const char *val)
 {
     plist_set_element_val(node, PLIST_STRING, val, strlen(val));
 }
 
-void plist_set_bool_val(plist_t node, uint8_t val)
+PLIST_API void plist_set_bool_val(plist_t node, uint8_t val)
 {
     plist_set_element_val(node, PLIST_BOOLEAN, &val, sizeof(uint8_t));
 }
 
-void plist_set_uint_val(plist_t node, uint64_t val)
+PLIST_API void plist_set_uint_val(plist_t node, uint64_t val)
 {
     plist_set_element_val(node, PLIST_UINT, &val, sizeof(uint64_t));
 }
 
-void plist_set_uid_val(plist_t node, uint64_t val)
+PLIST_API void plist_set_uid_val(plist_t node, uint64_t val)
 {
     plist_set_element_val(node, PLIST_UID, &val, sizeof(uint64_t));
 }
 
-void plist_set_real_val(plist_t node, double val)
+PLIST_API void plist_set_real_val(plist_t node, double val)
 {
     plist_set_element_val(node, PLIST_REAL, &val, sizeof(double));
 }
 
-void plist_set_data_val(plist_t node, const char *val, uint64_t length)
+PLIST_API void plist_set_data_val(plist_t node, const char *val, uint64_t length)
 {
     plist_set_element_val(node, PLIST_DATA, val, length);
 }
 
-void plist_set_date_val(plist_t node, int32_t sec, int32_t usec)
+PLIST_API void plist_set_date_val(plist_t node, int32_t sec, int32_t usec)
 {
     struct timeval val = { sec, usec };
     plist_set_element_val(node, PLIST_DATE, &val, sizeof(struct timeval));
